@@ -1,16 +1,30 @@
-  package Package::Subroutine
-# ***************************
-; our $VERSION = '0.16_3'
-# *********************
+package Package::Subroutine;
+# **************************
+$VERSION = '0.21.1';
+# ****************
+; no strict 'refs'
+
+; use Class::ISA ()
+
 ; sub export_to_caller
     { my ($self,$level) = @_
     ; my $namespace = (caller($level))[0]
     ; return sub
-      { my ($to,@methods) = @_
-      ; $to = caller if $to eq '_'
-      ; exporter($namespace,$to,@methods)
+      { my ($from,@methods) = @_
+      ; $from = caller if $from eq '_'
+      ; exporter($namespace,$from,@methods)
       }
     }
+
+; sub export_to
+    { my ($self,$namespace) = @_
+    ; return sub
+        { my ($from,@methods) = @_
+        ; $from = caller if $from eq '_'
+        ; exporter($namespace,$from,@methods)
+        }
+    }
+
 ; sub export
     { my $ns = (caller(1))[0]
     ; shift() # rm package
@@ -29,8 +43,8 @@
     { my $ns  = (caller(0))[0]
     ; my $pkg = shift()
     ; if(@_==1)
-	{ push @_, $pkg->findsubs($ns)
-	}
+        { push @_, $pkg->findsubs($_[0])
+        }
     ; exporter($ns,@_)
     }
 
@@ -41,21 +55,20 @@
     ; local $_
 
     ; for ( @methods )
-	{ my $srcm = my $trgm = $_
-	; ($srcm,$trgm) = @$_ if ref eq 'ARRAY'
-
-	; my $target = "${namespace}::${trgm}"
-	; my $source = "${from}::${srcm}"
-	; *$target = \&$source
-	}
+        { my $srcm = my $trgm = $_
+        ; ($srcm,$trgm) = @$_ if ref eq 'ARRAY'
+        ; my $target = "${namespace}::${trgm}"
+        ; my $source = "${from}::${srcm}"
+        ; *$target = \&$source
+        }
     }
 
 ; sub version
     { my ($f,$pkg,$num)=@_
     ; if( defined($num) )
-	{ $num=eval { UNIVERSAL::VERSION($pkg,$num) }
-	; return $@ ? undef : $num
-	}
+        { $num=eval { UNIVERSAL::VERSION($pkg,$num) }
+        ; return $@ ? undef : $num
+        }
     ; eval { UNIVERSAL::VERSION($pkg) }
     }
 
@@ -80,6 +93,14 @@
    ; grep { *{"${class}::${_}"}{CODE} } keys %{"${class}::"}
    }
 
+; sub findmethods
+   { my ($self,$class)=@_
+   ; my %methods = map { $_ => 1 }
+       map { $self->findsubs($_) }
+       reverse  (Class::ISA::self_and_super_path($class),'UNIVERSAL')
+   ; return keys %methods
+   }
+
 ; 1
 
 __END__
@@ -151,8 +172,8 @@ ecological niche.
 A possible use case for this module is an situation where a package
 decides during load time from where the used functions come from.
 In such a case Exporter is not a good solution because it
-is bound to C<use> and C<@ISA> what made things a little bit
-harder to change things dynamically.
+is bound to C<use> and C<@ISA> what makes things a little bit
+harder to change.
 
 The inport or export needs at least two arguments. The first is a
 package name. Second argument is a list of function names.
@@ -161,7 +182,7 @@ It is safest, if the package was loaded before you transfer the subs
 around.
 
 There is a shortcut for the current namespace included because
-you can't write
+you shouldn't write
 
    export Package::Subroutine __PACKAGE__ => qw/foo bar/
 
@@ -189,10 +210,17 @@ of method names imports all subs from the given namespace.
 
 =head2 C<export_to_caller>
 
-This method takes the level for the caller function call and return a code
-reference which wraps C<exporter> function curried with the specified
+This method takes the level for the caller function call and returns a code
+reference which wraps the C<exporter> function curried with the specified
 target namespace.
 
+=head2 C<export_to>
+
+The right tool to export subroutines into an arbitrary namespace. The
+argument here is a package name, the target for the export. It works like
+C<export_to_caller> and returns a code reference. These should be called with
+the source namespace and the subroutine names.
+
 =head2 C<exporter>
 
 The methods above are using one function. This function is usable with full
@@ -227,6 +255,11 @@ or a pair of package name and function name.
 This method returns a list or an array with all defined functions
 for a given package.
 
+=head2 C<findmethods>
+
+Simliar to findsubs but reads all classes in C<@ISA> plus UNIVERSAL.
+L<Class::ISA> is used for the task to list all subclasses.
+
 =head1 Note
 
 I know this package does not much, what is not possible with core
@@ -252,6 +285,10 @@ readable.
 
 =item L<Exporter>
 
+=item L<Class::Inspector>
+
+The C<methods> method has much more features than findmethods.
+
 =back
 
 =head1 CONTRIBUTIONS
@@ -265,4 +302,5 @@ should happen anyway.)
 Perl has a free license, so this module shares it with this
 programming language.
 
-Copyleft 2006-2009 by Sebastian Knapp E<lt>rock@ccls-online.deE<gt>
+Copyleft 2006-2012 by Sebastian Knapp E<lt>rock@ccls-online.deE<gt>
+
